% 1D FDTD antireflection layer for a antenna to receive at 2.4 GHz
%
% Lucijan Ilic
% KF Uni Graz
% 01530009

%% Constants (si system)
c0 = physconst('LightSpeed');
e0 = 8.85418781762e-12;
u0 = 4*pi*10^(-7);

%% FTDT parameters
%Grid
N_lam = 20; % shortest wavelength resolved with Nlam points
N_space = [50 50]; % Points before after the device 

% Source
f_max = 5e9; % 5GHz % max. simulated freq
N_f = 1000; % 1000 Frequencies from 0 to 5Ghz --> For Fourier Trafo
f = linspace(0,f_max,N_f); % Frequencies

%Device parameters
er_air = 1;
ur_air = 1;
d_device1 = 0.30; % 30 cm (plastic coating to protect the antenna)
er_device1 = 12; % plastic coating to protect the antenna
ur_device1 = 1;

%Calculate properities for the antireflection grating
f_ant = 2.4e9; % 2.4 GHz antenna
er_device2 = sqrt(er_air*er_device1); % geometric mean to approx the permitivity
ur_device2 = 1;
n_antirefl = sqrt(er_device2*ur_device2);
lam_0 = c0/f_ant; % air wavelength at 2.4GHz
lam_antirefl = lam_0/n_antirefl;
d_device2 = lam_antirefl/(4); % thickness of antireflection layer
ur_device2 = 1;

n_max = sqrt(max([er_air er_device1 er_device2])*max([ur_air ur_device1 ur_device2])); % Maximum Brechungsindex
N_dim = 1; % resolve minimum dimension with at least N_dim grid cells
dz1 = d_device1/N_dim; % grid resolution based on resolving the min. dimension (middle layer of the device)
dz2 = d_device2/N_dim; % grid resolution based on resolving the min. dimension (outer layers of the device)

% Compute parameters
lam_min = c0/(n_max*f_max); % shortest wavelength in simulation
dz3 = lam_min/N_lam; % grid resolution base on resolving min. wavelength
dz = min([dz1 dz2 dz3]); % go with the smallest grid resolution
N_device2 = ceil(d_device2/dz); % how many grid cells the outer layers of the device needs (round up)
dz = d_device2/N_device2; % adjust to critical dimension of the smallest layer of the device
N_device1 = ceil(d_device1/dz); % how many grid cells the middle layer of the device needs (round up)
Nz = N_device1 + 2*N_device2 + sum(N_space) + 3; % Grid size (+3 for Transmission, Reflection and Source)
z = [0:Nz-1]*dz; % Grid axis


%% Device parameters 
er = er_air*ones(1,Nz);
ur = ur_air*ones(1,Nz);

% Layer1
n_start1 = 2 + N_space(1); % start point of the first layer on the grid
n_end1 = n_start1 + N_device2 - 1; % end point
er(n_start1:n_end1) = er_device2; % update permitivity due to the injection of the first layer
ur(n_start1:n_end1) = ur_device2;

% Layer2
n_start2 = n_end1 + 1; % start point of the second layer on the grid
n_end2 = n_start2 + N_device1 - 1; % end point
er(n_start2:n_end2) = er_device1; % update permitivity due to the injection of the sec. layer
ur(n_start2:n_end2) = ur_device1;

% Layer3
n_start3 = n_end2 + 1; % start point of the third layer on the grid
n_end3 = n_start3 + N_device2 - 1; % end point
er(n_start3:n_end3) = er_device2; % update permitivity due to the injection of the third layer
ur(n_start3:n_end3) = ur_device2;

%% Compute the source
n_bc = sqrt(er(1)*ur(1)); % refracting index at the boundary (because we need it for PABC)
dt = n_bc*dz/(c0); % Time resolution/step (this value because we need to satisfy PABC)

tau = 0.5/f_max; % Parameter for the gaussian (approx. with Fourier Transform)
t0 = 5*tau; % we need to delay the gaussian otherwise we would start directly with high values

t_travel = n_max*Nz*dz/c0; % Time that it takes the pulse to travel trough the grid
T_sim = 2*t0 + 7*t_travel; % Total simulation time 
                      % 2 offsets to get in and out of the source and let the source travel 3 times trough the grid
Nt = ceil(T_sim/dt); % Total number of time steps

t = [0:Nt]*dt; % Time axis
nz_inj = 2; %inject the source 2nd place in the grid (1st for reflection)

% Injected source
E_inj = exp(-((t-t0)/tau).^2); % Amplitude = 1
A = -sqrt(er(nz_inj)/ur(nz_inj)); % Amlitude for H injection corresponding to the algorithm
delta_t = dz/(2*c0) + dt/2; % delta_t is the delay to start simultan. with E (due to the Grid construction)
                            % H and E are halb time step and halb grid step apart
H_inj = A*exp(-((t-t0+delta_t)/tau).^2); 

%% Fields and update ancoefficients and other parameters (Fourier)
% Using the update equation of the Ey/Hx mode
c_Ey = (c0*dt)./er;
c_Hx = (c0*dt)./ur;

Ey = zeros(1,Nz);
Hx = zeros(1,Nz);

%For boundary condtions
H1=0; H2=0;
E1=0; E2=0;

% Fourier Trafo
K = exp(-1i*2*pi*dt*f); %Kernel of the Fourier Trafo for each freq.
re = zeros(1,N_f); % Reflection R
tr = zeros(1,N_f); % Transmission T
re_tr = zeros(1,N_f); % R+T
source = zeros(1,N_f); % to normalize reflection and transmission later on 
                       % (because not every freq of the source delivers the same amount of energy)

%% FDTD Simulation --> updating the fields
figure('Color', 'White');

for t = 1:Nt
   for nz = 1:Nz-1 % run trough every grid point in the z axes
       Hx(nz) = Hx(nz) + c_Hx(nz)*(Ey(nz+1) - Ey(nz))/dz;
   end
   Hx(Nz) = Hx(Nz) + c_Hx(Nz)*(E2 - Ey(Nz))/dz; % Perfectly Absorbing BC
   
   % H source (TF/SF)
   Hx(nz_inj-1) = Hx(nz_inj-1) - c_Hx(nz_inj-1)*E_inj(t)/dz; % incl. Correction term for TF/SF
   
   H2=H1; H1=Hx(1); % for the boundary condtitions because the value travels one grid step in two time steps
   
   Ey(1) = Ey(1) + c_Ey(1)*(Hx(1) - H2)/dz; % Perfectly Absorbing BC
   for nz = 2:Nz 
       Ey(nz) = Ey(nz) + c_Ey(nz)*(Hx(nz) - Hx(nz-1))/dz;
   end
   
   % E source (TF/SF)
   Ey(nz_inj) = Ey(nz_inj) - c_Ey(nz_inj)*H_inj(t)/dz; % incl. Correction term for TF/SF
   
   E2=E1; E1=Ey(Nz); % for the boundary condtitions because the value travels on grid step in two time steps
   
  
   %Fourier Trafo
   for i = 1:N_f
      re(i) = re(i) + K(i)^(t)*Ey(1); % storing the the refelcted amount with the first cell in each time step
      tr(i) = tr(i) + K(i)^(t)*Ey(Nz); % storing the the transmitted amount with the last cell in each time step
      source(i) = source(i) + K(i)^(t)*E_inj(t); % store the source for later normalization of R and T
   end
   
   % Plot updated fields every ... iteratition
   if ~mod(t,500)
       subplot(2,1,1); % -> Simulation
       plotDeviceFields(er,dz,Ey,Hx); % Draw device (corresponding to the permitivity vektor) and the fields
       xlim([0 (Nz-1)*dz]);
       y_max = max([abs(min([Ey Hx])) max([Ey Hx])]); % save for scalling the y axis
       ylim([-(y_max+0.1*y_max) (y_max+0.1*y_max)]);
       xlabel('z');
       title(['Step ' num2str(t)]);
       
       % Plot R and T
       R = abs(re./source).^2;
       T = abs(tr./source).^2;
       subplot(2,1,2); % -> Reflection and Transmission
       plot(f,R,'-r',f,T,'-b',f,R+T,'-k');
       xlim([f(1) f(N_f)]);
       ylim([-0.2 1.8]);
       xlabel('f(Hz)');
       title('Reflection and Transmission');
       legend({'Reflection R','Transmission T','R+T'},'Location','north')
       drawnow;
   end
end

% Normalize Reflection and Transmission --> Final results after the simulation
%R_final = abs(re./source).^2;
%T_final = abs(tr./source).^2;




