% 2D FDTD 
% PML
% Lucijan Ilic
% KF Uni Graz
% 01530009

%% Constants (si system)
c0 = physconst('LightSpeed');
e0 = 8.85418781762e-12;
u0 = 4*pi*10^(-7);

%% FTDT parameters
Nx = 100;
Ny = 100;
dx = 0.11;
dy = 0.11;
dt = 1.667e-10;
f_0 = 150e6;
tau = 1/(1.5*f_0); % parameter for the gaussian
Nt = 400;

% Grid axis
x = [0:Nx-1]*dx;
y = [0:Ny-1]*dy;
%% Device parameters -> here just vacuum/air (diagonal anisotropic)
er_xx = ones(Nx,Ny);
ur_xx = ones(Nx,Ny);
er_yy = ones(Nx,Ny);
ur_yy = ones(Nx,Ny);
er_zz = ones(Nx,Ny);
ur_zz = ones(Nx,Ny);

%% Compute the sourc
%inject the source somewhere in the grid
x_inj = round(Nx/2);
y_inj = round(Ny/2);

t = [0:Nt]*dt; % Time axis
t0 = 4*tau; % we need to delay the gaussian otherwise we would start directly with high values

% Injected source
Dz_inj = exp(-2*((t-t0)/tau).^2).*sin(2*pi*t*f_0); 

%% Fields, Integrals and curls
Ez = zeros(Nx,Ny);
Dz = zeros(Nx,Ny);
Hx = zeros(Nx,Ny);
Hy = zeros(Nx,Ny);

C_Ex = zeros(Nx,Ny);
C_Ey = zeros(Nx,Ny);
C_Hz = zeros(Nx,Ny);

I_C_Ex = zeros(Nx,Ny);
I_C_Ey = zeros(Nx,Ny);
I_Dz = zeros(Nx,Ny);

%% PML Parameters and 2x grid
% Create 2x grid to handle the problem with the staggered fields:
Nx_2 = 2*Nx;
Ny_2 = 2*Ny;
N_pml = 9; % pml boundary

%PML parameters (loss in the outer edge of the 2D grid)
sig_x = zeros(Nx_2,Ny_2);
sig_y = zeros(Nx_2,Ny_2);
%left side
for i = 1:2*N_pml
    ix = 2*N_pml - i + 1;
    sig_x(ix,:) = e0/(2*dt)*(i/(2*N_pml))^3;
    ix = Nx_2 - 2*N_pml + i;
    sig_x(ix,:) = e0/(2*dt)*(i/(2*N_pml))^3;
    iy = 2*N_pml - i + 1;
    sig_y(:,iy) = e0/(2*dt)*(i/(2*N_pml))^3;
    iy = Ny_2 - 2*N_pml + i;
    sig_y(:,iy) = e0/(2*dt)*(i/(2*N_pml))^3;
end
sig_x = sig_x*1.9;
sig_y = sig_y*1.9;
%% Update coefficients
sigHx = sig_x(1:2:Nx_2,2:2:Ny_2);
sigHy = sig_y(1:2:Nx_2,2:2:Ny_2);
a_Hx1 = (1/dt - sigHy/(2*e0))./(1/dt + sigHy/(2*e0));
a_Hx2 = -(c0./ur_xx)./(1/dt + sigHy/(2*e0));
a_Hx3 = -(c0*dt*sigHx./(e0*ur_xx))./(1/dt + sigHy/(2*e0));

sigHx = sig_x(2:2:Nx_2,1:2:Ny_2);
sigHy = sig_y(2:2:Nx_2,1:2:Ny_2);
a_Hy1 = (1/dt - sigHx/(2*e0))./(1/dt + sigHx/(2*e0));
a_Hy2 = -(c0./ur_yy)./(1/dt + sigHx/(2*e0));
a_Hy3 = -(c0*dt*sigHy./(e0*ur_yy))./(1/dt + sigHx/(2*e0));

sigDx = sig_x(1:2:Nx_2,1:2:Ny_2);
sigDy = sig_y(1:2:Nx_2,1:2:Ny_2);
a_Dz1 = (1/dt - (sigDx+sigDy)/(2*e0) - dt*(sigDx.*sigDy)/(4*(e0^2)))./(1/dt + (sigDx+sigDy)/(2*e0) + dt*(sigDx.*sigDy)/(4*(e0^2)));
a_Dz2 = c0./(1/dt + (sigDx+sigDy)/(2*e0) + dt*(sigDx.*sigDy)/(4*(e0^2)));
a_Dz3 = -(dt*(sigDx.*sigDy)/(e0^2))./(1/dt + (sigDx+sigDy)/(2*e0) + dt*(sigDx.*sigDy)/(4*(e0^2)));

%% Pre Plot
fig = pcolor(x,y,Ez);
xlabel( 'x' );
ylabel( 'y' );
shading interp;
axis equal tight;
colormap('jet');
caxis([-0.1 0.1])
%colorbar();

%% Time iteration
for it = 2:Nt
    %% Calculation of the E curl equations
    % C_Ex
    for ix = 1:Nx
        for iy = 1:Ny-1
            C_Ex(ix,iy) = (Ez(ix,iy+1)-Ez(ix,iy))/dy;
        end
        C_Ex(ix,Ny) = (0-Ez(ix,Ny))/dy; %Dirichlet B.C.
    end
    
    %C_Ey
    for iy = 1:Ny
        for ix = 1:Nx-1
            C_Ey(ix,iy) = -(Ez(ix+1,iy)-Ez(ix,iy))/dx;
        end
        C_Ey(Nx,iy) = -(0-Ez(Nx,iy))/dx; %Dirichlet B.C.
    end
    %% Update I_C_Ex and I_C_Ey (integrals)
    I_C_Ex = I_C_Ex + C_Ex;
    I_C_Ey = I_C_Ey + C_Ey;
    %% Update H equations
    for ix = 1:Nx
        for iy = 1:Ny
            Hx(ix,iy) = a_Hx1(ix,iy)*Hx(ix,iy) + a_Hx2(ix,iy)*C_Ex(ix,iy) + a_Hx3(ix,iy)*I_C_Ex(ix,iy);
            Hy(ix,iy) = a_Hy1(ix,iy)*Hy(ix,iy) + a_Hy2(ix,iy)*C_Ey(ix,iy) + a_Hy3(ix,iy)*I_C_Ey(ix,iy);
        end
    end
    %% Calculation of the H curl equation
    %C_Hz
    C_Hz(1,1) = (Hy(1,1)-0)/dx-(Hx(1,1)-0)/dy;
    for ix = 2:Nx
        C_Hz(ix,1) = (Hy(ix,1)-Hy(ix-1,1))/dx-(Hx(ix,1)-0)/dy;
    end
    for iy = 2:Ny
        C_Hz(1,iy) = (Hy(1,iy)-0)/dx-(Hx(1,iy)-Hx(1,iy-1))/dy;
        for ix = 2:Nx
            C_Hz(ix,iy) = (Hy(ix,iy)-Hy(ix-1,iy))/dx-(Hx(ix,iy)-Hx(ix,iy-1))/dy;
        end
    end
    %% Update I_Dz (integral)
    I_Dz = I_Dz + Dz;
    %% Update D equations
    for ix = 1:Nx
        for iy = 1:Ny
            Dz(ix,iy) = a_Dz1(ix,iy)*Dz(ix,iy) + a_Dz2(ix,iy)*C_Hz(ix,iy) + a_Dz3(ix,iy)*I_Dz(ix,iy);
        end
    end
    %% Implement/update the source
    Dz(x_inj,y_inj) = Dz(x_inj,y_inj) + Dz_inj(it);
    
    %% Derive E from D
    for ix = 1:Nx
        for iy = 1:Ny
            Ez(ix,iy) = Dz(ix,iy)/(er_zz(ix,iy));
        end
    end
    
    %% Plot Ez field
    if ~mod(it,1)
        set(fig,'CData',Ez);
        drawnow;
    end
end
