% 2D FDTD 
% Peridodic boundary condition
% Lucijan Ilic
% KF Uni Graz
% 01530009

%% Constants (si system)
c0 = physconst('LightSpeed');
e0 = 8.85418781762e-12;
u0 = 4*pi*10^(-7);

%% FTDT parameters
Nx = 100;
Ny = 100;
dx = 0.11;
dy = 0.11;
dt = 1.667e-10;
f_0 = 150e6;
tau = 1/(1.5*f_0); % parameter for the gaussian
Nt = 400;

% Grid axis
x = [0:Nx-1]*dx;
y = [0:Ny-1]*dy;

%% Compute the sourc
%inject the source somewhere in the grid
x_inj = round(Nx/3);
y_inj = round(Ny/3);

t = [0:Nt]*dt; % Time axis
t0 = 4*tau; % we need to delay the gaussian otherwise we would start directly with high values

% Injected source
Dz_inj = exp(-2*((t-t0)/tau).^2).*sin(2*pi*t*f_0); 

%% Device parameters -> here just vacuum/air
er = ones(Nx,Ny);
ur = ones(Nx,Ny);

%% Fields, update coefficients and curls
a_Hy = -(c0*dt)./ur;
a_Hx = -(c0*dt)./ur;
a_Dz = c0*dt;

Ez = zeros(Nx,Ny);
Dz = zeros(Nx,Ny);
Hx = zeros(Nx,Ny);
Hy = zeros(Nx,Ny);

C_Ex = zeros(Nx,Ny);
C_Ey = zeros(Nx,Ny);
C_Hz = zeros(Nx,Ny);

%% Pre Plot
fig = pcolor(x,y,Ez);
xlabel( 'x' );
ylabel( 'y' );
shading interp;
axis equal tight;
caxis([-0.06 0.06])
%colorbar();
colormap('jet');

%% Time iteration
for it = 2:Nt
    %% Calculation of the E curl equations
    % C_Ex
    for ix = 1:Nx
        for iy = 1:Ny-1
            C_Ex(ix,iy) = (Ez(ix,iy+1)-Ez(ix,iy))/dy;
        end
        C_Ex(ix,Ny) = (Ez(ix,1)-Ez(ix,Ny))/dy; %Dirichlet B.C.
    end
    
    %C_Ey
    for iy = 1:Ny
        for ix = 1:Nx-1
            C_Ey(ix,iy) = -(Ez(ix+1,iy)-Ez(ix,iy))/dx;
        end
        C_Ey(Nx,iy) = -(Ez(1,iy)-Ez(Nx,iy))/dx; %Dirichlet B.C.
    end
    
    %% Update H equations
    for ix = 1:Nx
        for iy = 1:Ny
            Hx(ix,iy) = Hx(ix,iy) + a_Hx(ix,iy)*C_Ex(ix,iy);
            Hy(ix,iy) = Hy(ix,iy) + a_Hy(ix,iy)*C_Ey(ix,iy);
        end
    end
    %% Calculation of the H curl equation
    %C_Hz
    C_Hz(1,1) = (Hy(1,1)-Hy(Nx,1))/dx-(Hx(1,1)-Hx(1,Ny))/dy;
    for ix = 2:Nx
        C_Hz(ix,1) = (Hy(ix,1)-Hy(ix-1,1))/dx-(Hx(ix,1)-Hx(ix,Ny))/dy;
    end
    for iy = 2:Ny
        C_Hz(1,iy) = (Hy(1,iy)-Hy(Nx,iy))/dx-(Hx(1,iy)-Hx(1,iy-1))/dy;
        for ix = 2:Nx
            C_Hz(ix,iy) = (Hy(ix,iy)-Hy(ix-1,iy))/dx-(Hx(ix,iy)-Hx(ix,iy-1))/dy;
        end
    end
    
    %% Update D equations
    for ix = 1:Nx
        for iy = 1:Ny
            Dz(ix,iy) = Dz(ix,iy) + a_Dz*C_Hz(ix,iy);
        end
    end
    %% Implement/update the source
    Dz(x_inj,y_inj) = Dz(x_inj,y_inj) + Dz_inj(it);
    %% Derive E from D
    for ix = 1:Nx
        for iy = 1:Ny
            Ez(ix,iy) = Dz(ix,iy)/(er(ix,iy));
        end
    end
    
    %% Plot Ez field (er=1 --> Ez=Dz)
    if ~mod(it,1)
        set(fig,'CData',Ez);
        drawnow;
    end
end
